const _ = require("lodash");
const axios = require('axios');
const url = require('url');

const OP = {
    "gt": ">",
    "lt": "<",
    "ne": "not in",
    "eq": "in"
}

class KowalskiAPI {

    post(endpoint, objects){
        let kwObjects = this.map(objects);
        return this._doPost(endpoint, kwObjects);
    }

    map(objects){
        let kwObjects = {};

        _.extend(kwObjects, this._map(objects, "datasources", this._mapDatasources))
        _.extend(kwObjects, this._map(objects, "datasets", this._mapDatasets))
        _.extend(kwObjects, this._map(objects, "charts", this._mapCharts))
        _.extend(kwObjects, this._map(objects, "reports", this._mapReports))
        _.extend(kwObjects, this._map(objects, "dashboards", this._mapDashboards))

        return kwObjects;
    }

    _doPost(endpoint, kwObjects){
        return axios.post(endpoint + "/api/push", kwObjects);
    }

    _mapDatasources (d, key) {
        return {
            code: key,
            type: d.type,
            options: {
                url: d.url,
                user: d.user,
                password: d.password,
                dialect: d.dialect
            }
        }
    }

    _mapDatasets(d, key)  {
        let columns = _.map(d.columns, (col, key) => {
            let type = col[0]
            let typeName = "";
            let typeSpec =  "";

            let init = type.indexOf('(');
            let end = type.indexOf(')');

            if(init >= 0 && end >= 0){
                typeName = type.substring(0, init);
                typeSpec = type.substring(init + 1, end);
            } else {
                typeName = type;
            }

            let mappedCol = {
                name: key,
                description: col[1]
            };

            switch(typeName){
                case 'text':
                    mappedCol['type'] = 12;

                    let length = 255;
                    if(typeSpec){
                        mappedCol['length'] = parseInt(typeSpec)
                    }

                    mappedCol['length'] = length;

                    break;

                case 'number':
                    mappedCol['type'] = 2;

                    let scale = 0;
                    let precision = 19;

                    if(typeSpec && typeSpec.indexOf(',') >= 0){
                        precision = parseInt(typeSpec.split(",")[0]);
                        scale = parseInt(typeSpec.split(",")[1]);
                    }

                    mappedCol['scale'] = scale;
                    mappedCol['precision'] = precision;
                    break;

                case 'date':
                    mappedCol['type'] = 93;
                    break;
            }


            return mappedCol;
        });

        return {
            codigo: key,
            query: d.query,
            replicate: d.replicate,
            name: d.name,
            description: d.description,
            datasource: d.datasource,
            columns: columns
        };
    }



    _mapCharts(d, key){
        let chart = {
            code: key,
            dataset: d.data.dataset,
            chartType: d.options.type,
            name: d.options.title
        }

        chart["dimensions"] = _.map(d.data.mapping, (m, mKey) => {
            let dimension = {};

            if(mKey === 'Colunas' && m instanceof Array){
              let mappedColumns = [];
              _.each(m, (item) => {

                let mappedColumn = {
                    name: item,
                    alias: item
                  }

                  if(d.options.format && d.options.format[item]){
                    mappedColumn["options"] = {
                      formatter: d.options.format[item]
                    }
                  }

                  if(d.options.aggregation && d.options.aggregation[item]){
                    dimension["aggregation"] = d.options.aggregation[item]
                  }

                  mappedColumns.push(mappedColumn);
              });

              dimension = {
                mappedColumns: mappedColumns,
                name: mKey
              }

            } else if(mKey === 'Group' && m instanceof Array){
                let mappedColumns = [];
                _.each(m, (item) => {

                  let mappedColumn = {
                      name: item,
                      alias: item
                    }

                    if(d.options.format && d.options.format[item]){
                      mappedColumn["options"] = {
                        formatter: d.options.format[item]
                      }
                    }

                    if(d.options.aggregation && d.options.aggregation[item]){
                      dimension["aggregation"] = d.options.aggregation[item]
                    }

                    mappedColumns.push(mappedColumn);
                });

                dimension = {
                  mappedColumns: mappedColumns,
                  name: mKey
                }

              } else {

              let mappedColumn = {
                name: m,
                alias: m
              }

              if(d.options.format && d.options.format[m]){
                mappedColumn["options"] = {
                  formatter: d.options.format[m]
                }
              }

              dimension = {
                mappedColumns: [mappedColumn],
                name: mKey
              }

              if(d.options.aggregation && d.options.aggregation[m]){
                dimension["aggregation"] = d.options.aggregation[m]
              }

            }

            return dimension;
        });

        if(d.data.restrictions) {
            chart["dataRestrictions"] = _.mapValues(d.data.restrictions, (r, rKey) => {
                return _.map(r, (val, op) => {
                    return {"op": OP[op], "value": val}
                });
            });
        }

        let mappedOptions = _.omit(d.options,
            "title", "type", "format", "aggregation", "margin", "link", "cutMark", "legend", "colorMapping");

        let margin = {};
        if(d.options.margin){
            let marginSplitted = d.options.margin.split(" ");
            margin = {
                top: marginSplitted[0],
                right: marginSplitted[1],
                bottom: marginSplitted[2],
                left: marginSplitted[3]
            }

            mappedOptions["margin"] = margin;
        }

        if(d.options.link){
            let linkColumn = {};

            var isColumnDynamic = (!d.options.link.columName) ? true : false;
            linkColumn = {
                alias: (d.options.link.label === undefined) ? '' : d.options.link.label,
                name: (d.options.link.columnLabel === undefined) ? '' : d.options.link.columnLabel,
                columName: isColumnDynamic ? d.options.link.queryString : d.options.link.columName ,
                isDynamic: isColumnDynamic,
                url: d.options.link.url
            }

            mappedOptions["linkColumn"] = linkColumn;
        }

        let cutMark = {}
        if(d.options.cutMark){
            cutMark = {
                value: d.options.cutMark.value,
                label: d.options.cutMark.description,
                aboveColor: d.options.cutMark.colorAbove,
                belowColor: d.options.cutMark.colorBelow
            }

            mappedOptions["cutMark"] = cutMark;
        }

        if(d.options.legend){
            mappedOptions["legend"] = d.options.legend;
        }

        if(d.options.colorMapping){
             mappedOptions["colors"] = d.options.colorMapping;
        }

        if(!_.isEmpty(_.keys(mappedOptions))) {
            chart["options"] = mappedOptions;
        }

        return chart;
    }

    _mapReports(d, key){

        let items = _.map(d.navigation, function(nav, index){
            return {
                chart: {code: nav},
                position: index,
                description: " "
            }
        })

        let _filters = [];
        _.each(d.requiredFilters, (item) => {
            _filters.push({"columnName": item, "alias": item, "required": true});
        });

        _.pullAll(d.filters, d.requiredFilters);
        _.each(d.filters, (item) => {
            _filters.push({"columnName": item, "alias": item});
        });

        return {
            code: key,
            title: d.title,
            dataset: d.dataset,
            items: items,
            availableFilterParameters: _filters
        }

    }

    _mapDashboards(d, key){
        let _grid = []
        _.map(d.grid, (row, index)=>{
            let cols = [];
            _.map(row, (col, index)=>{
                cols.push({'item': { 'code': col}});
            })
            _grid.push({'cols': cols});
        });

        let _filters = _.map(d.filters, (item, index)=>{
            return {"label": item, "value":item};
        });

        return {
            title: d.title,
            code: key,
            grid: _grid,
            filters: _filters
        }
    }

    _filterObjectsOfType(objects, type){
        return _.filter(objects, (o) => {
            return _.has(o, type);
        });
    }

    _map(objects, type, mapFn){
        let kwObject = {[type]: []};

        let objectsOfType = this._filterObjectsOfType(objects, type);

        _.each(objectsOfType, (obj) => {
            let mapped = _.map(obj[type], mapFn);
            _.each(mapped, (o) => {
                kwObject[type].push(o);
            });
        });

        return _.isEmpty(kwObject[type]) ? undefined : kwObject;
    }


}

module.exports = KowalskiAPI;